@@ -52,6 +52,7 @@
 #include <openssl/err.h>
 #include <openssl/evp.h>
 #include <openssl/hmac.h>
+#include <openssl/rsa.h>
 #include "capture.h"
 
 //! Cast two bytes into decimal (Big Endian)
@@ -103,6 +104,25 @@ enum SSLConnectionState {
     TCP_STATE_CLOSED
 };
 
+//! SSL Encoders algo
+enum SSLCipherEncoders {
+    ENC_AES         = 1,
+    ENC_AES256      = 2
+};
+
+//! SSL Digests algo
+enum SSLCIpherDigest {
+    DIG_SHA1        = 1,
+    DIG_SHA256      = 2,
+    DIG_SHA384      = 3
+};
+
+//! SSL Decode mode
+enum SSLCipherMode {
+    MODE_CBC,
+    MODE_GCM
+};
+
 //! ContentType values as defined in RFC5246
 enum ContentType {
     change_cipher_spec = SSL3_RT_CHANGE_CIPHER_SPEC,
@@ -157,6 +177,16 @@ struct CipherSuite {
     uint8_t cs2;
 };
 
+struct CipherData {
+    int num;
+    int enc;
+    int ivblock;
+    int bits;
+    int digest;
+    int diglen;
+    int mode;
+};
+
 struct ClientHelloSSLv2 {
     struct ProtocolVersion client_version;
     uint16 cipherlist_len;
@@ -235,16 +265,17 @@ struct SSLConnection {
     struct Random client_random;
     struct Random server_random;
     struct CipherSuite cipher_suite;
+    struct CipherData cipher_data;
     struct PreMasterSecret pre_master_secret;
     struct MasterSecret master_secret;
 
     struct tls_data {
-        uint8_t client_write_MAC_key[20];
-        uint8_t server_write_MAC_key[20];
-        uint8_t client_write_key[32];
-        uint8_t server_write_key[32];
-        uint8_t client_write_IV[16];
-        uint8_t server_write_IV[16];
+        uint8_t *client_write_MAC_key;
+        uint8_t *server_write_MAC_key;
+        uint8_t *client_write_key;
+        uint8_t *server_write_key;
+        uint8_t *client_write_IV;
+        uint8_t *server_write_IV;
     } key_material;
 
     EVP_CIPHER_CTX *client_cipher_ctx;
@@ -273,24 +304,6 @@ int
 P_hash(const char *digest, unsigned char *dest, int dlen, unsigned char *secret, int sslen,
        unsigned char *seed, int slen);
 
-/**
- * @brief Pseudorandom Function as defined in RFC5246
- *
- * This function will generate MasterSecret and KeyMaterial data from PreMasterSecret and Seed
- *
- * @param dest Destination of PRF function result. Memory must be already allocated
- * @param dlen Destination length in bytes
- * @param pre_master_secret PreMasterSecret decrypted from ClientKeyExchange Handhsake record
- * @param pslen PreMasterSecret length in bytes
- * @param label Fixed ASCII string
- * @param seed Concatenation of Random data from Hello Handshake records
- * @param slen Seed length in bytes
- * @return destination length in bytes
- */
-int
-PRF12(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
-        int plen, unsigned char *label, unsigned char *seed, int slen);
-
 /**
  * @brief Pseudorandom Function as defined in RFC2246
  *
@@ -306,8 +319,8 @@ PRF12(unsigned char *dest, int dlen, unsigned char *pre_master_secret,
  * @return destination length in bytes
  */
 int
-PRF(unsigned char *dest, int dlen, unsigned char *pre_master_secret, int plen, unsigned char *label,
-    unsigned char *seed, int slen);
+PRF(struct SSLConnection *conn, unsigned char *dest, int dlen, unsigned char *pre_master_secret,
+    int plen, unsigned char *label, unsigned char *seed, int slen);
 
 /**
  * @brief Create a new SSLConnection
