@@ -310,7 +310,7 @@ call_flow_draw_columns(ui_t *ui)
         } else if (setting_enabled(SETTING_DISPLAY_ALIAS)) {
             if (strlen(column->addr.ip) > 15) {
                 snprintf(coltext, MAX_SETTING_LEN, "..%.*s:%u",
-                         MAX_SETTING_LEN - 7, column->alias + strlen(column->alias) - 13, column->addr.port);
+                         MAX_SETTING_LEN - 9, column->alias + strlen(column->alias) - 13, column->addr.port);
             } else {
                 snprintf(coltext, MAX_SETTING_LEN, "%.*s:%u",
                          MAX_SETTING_LEN - 7, column->alias, column->addr.port);
@@ -318,7 +318,7 @@ call_flow_draw_columns(ui_t *ui)
         } else {
             if (strlen(column->addr.ip) > 15) {
                 snprintf(coltext, MAX_SETTING_LEN, "..%.*s:%u",
-                         MAX_SETTING_LEN - 7, column->addr.ip + strlen(column->addr.ip) - 13, column->addr.port);
+                         MAX_SETTING_LEN - 9, column->addr.ip + strlen(column->addr.ip) - 13, column->addr.port);
             } else {
                 snprintf(coltext, MAX_SETTING_LEN, "%.*s:%u",
                          MAX_SETTING_LEN - 7, column->addr.ip, column->addr.port);
@@ -465,12 +465,12 @@ call_flow_draw_message(ui_t *ui, call_flow_arrow_t *arrow, int cline)
     timeval_to_time(msg_get_time(msg), msg_time);
 
     // Get Message method (include extra info)
-    snprintf(method, METHOD_MAXLEN, "%s", msg_method);
+    snprintf(method, METHOD_MAXLEN, "%.*s", METHOD_MAXLEN-1, msg_method);
 
     // If message has sdp information
     if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "off")) {
         // Show sdp tag in title
-        snprintf(method, METHOD_MAXLEN, "%s (SDP)", msg_method );
+        snprintf(method, METHOD_MAXLEN, "%.*s (SDP)", METHOD_MAXLEN-7, msg_method );
     }
 
     // If message has sdp information
@@ -571,9 +571,9 @@ call_flow_draw_message(ui_t *ui, call_flow_arrow_t *arrow, int cline)
                     media->address.port,
                     media_get_prefered_format(media));
             if (arrow->dir == CF_ARROW_SPIRAL) {
-                mvwprintw(flow_win, cline + 1, startpos + 5, mediastr);
+                mvwprintw(flow_win, cline + 1, startpos + 5, "%s", mediastr);
             } else {
-                mvwprintw(flow_win, cline + 1, startpos + distance / 2 - strlen(mediastr) / 2 + 2, mediastr);
+                mvwprintw(flow_win, cline + 1, startpos + distance / 2 - strlen(mediastr) / 2 + 2, "%s", mediastr);
             }
             cline++;
             aline++;
@@ -1391,9 +1391,7 @@ call_flow_column_add(ui_t *ui, const char *callid, address_t addr)
     vector_append(column->callids, (void*)callid);
     column->addr = addr;
     if (setting_enabled(SETTING_ALIAS_PORT)) {
-        char addr_port[1024];
-        sprintf(addr_port, "%s:%d", addr.ip, addr.port);
-        strcpy(column->alias, get_alias_value(addr_port));
+        strcpy(column->alias, get_alias_value_vs_port(addr.ip, addr.port));
     } else {
         strcpy(column->alias, get_alias_value(addr.ip));
     }
@@ -1417,10 +1415,8 @@ call_flow_column_get(ui_t *ui, const char *callid, address_t addr)
     match_port = addr.port != 0;
 
     // Get alias value for given address
-    if (setting_enabled(SETTING_ALIAS_PORT)) {
-        char addr_port[1024];
-        sprintf(addr_port, "%s:%d", addr.ip, addr.port);
-        alias = get_alias_value(addr_port);
+    if (setting_enabled(SETTING_ALIAS_PORT) && match_port) {
+        alias = get_alias_value_vs_port(addr.ip, addr.port);
     } else {
         alias = get_alias_value(addr.ip);
     }
