@@ -86,8 +86,8 @@ filter_create(ui_t *ui)
     // Set fields options
     field_opts_off(info->fields[FLD_FILTER_SIPFROM], O_AUTOSKIP);
     field_opts_off(info->fields[FLD_FILTER_SIPTO], O_AUTOSKIP);
-    field_opts_off(info->fields[FLD_FILTER_SRC], O_AUTOSKIP);
-    field_opts_off(info->fields[FLD_FILTER_DST], O_AUTOSKIP);
+    field_opts_off(info->fields[FLD_FILTER_SRC], O_AUTOSKIP | O_STATIC);
+    field_opts_off(info->fields[FLD_FILTER_DST], O_AUTOSKIP | O_STATIC);
     field_opts_off(info->fields[FLD_FILTER_PAYLOAD], O_AUTOSKIP | O_STATIC);
     field_opts_off(info->fields[FLD_FILTER_REGISTER], O_AUTOSKIP);
     field_opts_off(info->fields[FLD_FILTER_INVITE], O_AUTOSKIP);
@@ -102,6 +102,10 @@ filter_create(ui_t *ui)
     field_opts_off(info->fields[FLD_FILTER_FILTER], O_EDIT);
     field_opts_off(info->fields[FLD_FILTER_CANCEL], O_EDIT);
 
+    // Set max field size
+    set_max_field(info->fields[FLD_FILTER_SRC], ADDRESSLEN);
+    set_max_field(info->fields[FLD_FILTER_DST], ADDRESSLEN);
+
     // Change background of input fields
     set_field_back(info->fields[FLD_FILTER_SIPFROM], A_UNDERLINE);
     set_field_back(info->fields[FLD_FILTER_SIPTO], A_UNDERLINE);
@@ -440,7 +444,7 @@ filter_field_method(int field_id)
 void
 filter_method_from_setting(const char *value)
 {
-    char methods[256], method_expr[256];
+    char methods[250], method_expr[256];
     int methods_len = strlen(value);
     char *comma;
 
@@ -455,7 +459,7 @@ filter_method_from_setting(const char *value)
 
         // Create a regular expression
         memset(method_expr, 0, sizeof(method_expr));
-        sprintf(method_expr, "(%s)", methods);
+        snprintf(method_expr, sizeof(method_expr), "(%s)", methods);
         filter_set(FILTER_METHOD, method_expr);
     } else {
         filter_set(FILTER_METHOD, " ");
